#region Copyright Syncfusion Inc. 2001-2016.
// Copyright Syncfusion Inc. 2001-2016. All rights reserved.
// Use of this code is subject to the terms of our license.
// A copy of the current license can be obtained at any time by e-mailing
// licensing@syncfusion.com. Any infringement will be prosecuted under
// applicable laws. 
#endregion
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
//using Xamarin.Forms;
using Windows.Storage;
using Windows.Storage.Pickers;
using ImageDrawing;
using ImageDrawing.UWP;
using Xamarin.Forms.Platform.UWP;
using Windows.UI.Xaml.Media.Imaging;
using Windows.Storage.Streams;
using Windows.Graphics.Imaging;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.UI.Xaml.Controls;

[assembly: Xamarin.Forms.Dependency(typeof(SaveWindows))]
namespace ImageDrawing.UWP
{
    class SaveWindows: ISave
    {

        public async Task SaveAsync(string filename, string contentType, MemoryStream stream)
        {
            if (Xamarin.Forms.Device.Idiom != Xamarin.Forms.TargetIdiom.Desktop)
            {
                StorageFolder local = Windows.Storage.ApplicationData.Current.LocalFolder;
                StorageFile outFile = await local.CreateFileAsync(filename, CreationCollisionOption.ReplaceExisting);
                using (Stream outStream = await outFile.OpenStreamForWriteAsync())
                {
                    outStream.Write(stream.ToArray(), 0, (int)stream.Length);
                }
                if (contentType != "application/html")
                    await Windows.System.Launcher.LaunchFileAsync(outFile);
            }
            else
            {
                StorageFile storageFile = null;
                FileSavePicker savePicker = new FileSavePicker();
                savePicker.SuggestedStartLocation = PickerLocationId.Desktop;
                savePicker.SuggestedFileName = filename;
                switch (contentType)
                {
                    case "application/vnd.openxmlformats-officedocument.presentationml.presentation":
                        savePicker.FileTypeChoices.Add("PowerPoint Presentation", new List<string>() { ".pptx", });
                        break;

                    case "application/msexcel":
                        savePicker.FileTypeChoices.Add("Excel Files", new List<string>() { ".xlsx", });
                        break;

                    case "application/msword":
                        savePicker.FileTypeChoices.Add("Word Document", new List<string>() { ".docx" });
                        break;

                    case "application/pdf":
                        savePicker.FileTypeChoices.Add("Adobe PDF Document", new List<string>() { ".pdf" });
                        break;
                    case "application/html":
                        savePicker.FileTypeChoices.Add("HTML Files", new List<string>() { ".html" });
                        break;
                }
                storageFile = await savePicker.PickSaveFileAsync();

                using (Stream outStream = await storageFile.OpenStreamForWriteAsync())
                {
                    outStream.Write(stream.ToArray(), 0, (int)stream.Length);
                    outStream.Flush();
                    outStream.Dispose();
                }
                stream.Flush();
                stream.Dispose();
                await Windows.System.Launcher.LaunchFileAsync(storageFile);
            }
        }
        public Stream ConvertPngToJpeg(Stream s)
        {
           return ConvertPngToJpeg2(s).Result;
        }

        public async Task<Stream> ConvertPngToJpeg2(Stream s)
        {

            byte[] resultArray = null;
            byte[] image = new byte[s.Length];
            s.Read(image, 0, image.Length);

            WriteableBitmap bitmapImage = new WriteableBitmap(1, 1);
            Canvas canvas = new Canvas();

            using (IRandomAccessStream ms = new InMemoryRandomAccessStream())
            {
                await ms.WriteAsync(image.AsBuffer());
                ms.Seek(0);
                bitmapImage.SetSource(ms);
                Image img = new Image();
                img.Source = bitmapImage;
                canvas.Children.Add(img);
                canvas.Width = bitmapImage.PixelWidth;
                canvas.Height = bitmapImage.PixelHeight;
                
            }

            RenderTargetBitmap renderTargetBitmap = new RenderTargetBitmap();
            await renderTargetBitmap.RenderAsync(canvas);

            var pixels = await renderTargetBitmap.GetPixelsAsync();

            using (IRandomAccessStream stream = new InMemoryRandomAccessStream())
            {
                var encoder = await BitmapEncoder.CreateAsync(BitmapEncoder.JpegEncoderId, stream);
                byte[] bytes = pixels.ToArray();
                encoder.SetPixelData(BitmapPixelFormat.Bgra8,
                                     BitmapAlphaMode.Ignore,
                                     (uint)canvas.Width, (uint)canvas.Height,
                                     96, 96, bytes);

                await encoder.FlushAsync();

                resultArray = new byte[stream.AsStream().Length];
                await stream.AsStream().ReadAsync(resultArray, 0, resultArray.Length);
            }

            Stream imgStream = new MemoryStream(resultArray);

            return imgStream;
        }
    }
}
